@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*               Copyright 2001-2002 by Gunnar Kedenburg                */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -227,7 +226,7 @@ namespace vigra
 
         // extract ICC profile
         JOCTET *iccBuf;
-        UInt32 iccLen;
+        unsigned int iccLen;
         if (read_icc_profile(&info, &iccBuf, &iccLen)) {
             iccProfileLength = iccLen;
             iccProfilePtr = iccBuf;
@@ -427,7 +426,7 @@ namespace vigra
         jpeg_start_compress( &info, TRUE );
 
         if (iccProfile.size()) {
-            write_icc_profile(&info, iccProfile.begin(), iccProfile.size());
+            write_icc_profile(&info, iccProfile.begin(), (unsigned int)iccProfile.size());
         }
     }
 
