@@ -48,6 +48,7 @@
 #include <vigra/random_forest_deprec.hxx>
 #include <unittest.hxx>
 #include <vector>
+#include <limits>
 //#include "data/RF_results.hxx"
 #include "data/RF_data.hxx"
 #include "test_visitors.hxx"
@@ -115,6 +116,247 @@ struct ClassifierTest
         std::ofstream newRFclear(newName.c_str());
         newRFclear.close();
     }
+    
+/** Tests whether RF rejects Matrices containing Inf
+ */	
+	void RF_InfCheck()
+    {
+        //Create Test output by Random Forest
+		typedef MultiArrayShape<2>::type Shp_t;
+        {
+            std::cerr << "RF_InfCheck(): checking contains_inf() and RF precondition on Inf\n";
+			MultiArray<2, double> dfeatures(Shp_t(10, 5));
+			MultiArray<2, float>  ffeatures(Shp_t(10, 5));
+			
+			for(int ii = 0; ii < 10; ++ii)
+			{
+				for(int jj = 0; jj < 5; ++jj)
+				{
+					dfeatures(ii, jj) = (ii*jj+jj)%97;
+					ffeatures(ii, jj) = (ii*jj+jj)%97;
+				}
+			}
+			shouldEqual(detail::contains_inf(dfeatures), false);
+			shouldEqual(detail::contains_inf(ffeatures), false);
+					
+			dfeatures(2, 3) = std::numeric_limits<double>::infinity();
+			ffeatures(2, 3) = std::numeric_limits<double>::infinity();
+			
+			shouldEqual(detail::contains_inf(dfeatures), true);
+			shouldEqual(detail::contains_inf(ffeatures), true);
+			
+			for(int ii = 0; ii < 10; ++ii)
+			{
+				for(int jj = 0; jj < 5; ++jj)
+				{
+					dfeatures(ii, jj) = std::numeric_limits<double>::infinity();
+					ffeatures(ii, jj) = std::numeric_limits<double>::infinity();
+				}
+			}
+			
+			shouldEqual(detail::contains_inf(dfeatures), true);
+			shouldEqual(detail::contains_inf(ffeatures), true);
+        }
+		MultiArray<2, double> dfeatures(Shp_t(10, 5));
+		MultiArray<2, double> dlabels(Shp_t(10, 1));
+		for(int ii = 0; ii < 10; ++ii)
+		{
+			for(int jj = 0; jj < 5; ++jj)
+			{
+				dfeatures(ii, jj) = (ii*jj+jj)%97;
+			}
+			dlabels(ii, 0) = ii%2;
+		}
+		dlabels(9,0) = std::numeric_limits<double>::infinity();
+		RandomForest<> rf;
+		try
+		{
+			rf.learn(dfeatures, dlabels);
+			shouldEqual(0, 1);
+		}
+		catch( ... )
+		{
+		}
+		dfeatures(9,0) = dlabels(9,0);
+		dlabels(9,0) = 9%2;
+		try
+		{
+			rf.learn(dfeatures, dlabels);
+			shouldEqual(0, 1);
+		}
+		catch( ... )
+		{
+		}
+
+    }
+	
+	void RF_AlgorithmTest()
+    {
+		std::cerr << "WARNING: THIS TEST CURRENTLY ONLY CHECKS WHETHER ALGORITHMS COMPILE AND RUN WITHOUT ERROR...\n";
+		int ii = data.size() - 3; // this is the pina_indians dataset
+		std::cerr << "RF_AlgorithmTest()....";
+		
+		rf::algorithms::HClustering				linkage;
+  		MultiArray<2, double>	distance;
+		std::cerr << "cluster_permutation_importance()....\n";
+  		cluster_permutation_importance(data.features(ii), data.labels(ii), linkage, distance);
+		rf::algorithms::VariableSelectionResult  result1;
+		std::cerr << "forward_selection()....\n";
+		rf::algorithms::forward_selection(data.features(ii), data.labels(ii), result1);
+		rf::algorithms::VariableSelectionResult  result;
+		std::vector<int> r;
+		r.push_back(0);
+		r.push_back(1);
+		r.push_back(2);
+		r.push_back(3);
+		r.push_back(4);
+		r.push_back(5);
+		r.push_back(6);
+		r.push_back(7);
+		result.init(data.features(ii), data.labels(ii), r.begin(), r.end());
+		std::cerr << "rank_selection()....\n";
+		rf::algorithms::rank_selection(data.features(ii), data.labels(ii), result);
+		
+		rf::algorithms::VariableSelectionResult  result2;
+		std::cerr << "backward_elimination()....\n";
+		rf::algorithms::backward_elimination(data.features(ii), data.labels(ii), result2);
+		std::cerr << "DONE\n";
+	}
+	void RF_SpliceTest()
+    {
+		std::cerr << "RF_SpliceTest()....";
+
+		MultiArray<2, double> a(MultiArrayShape<2>::type(20, 10), 1);
+		for(int ii = 0; ii< 20; ii+=2)
+		{
+			for(int jj = 0; jj< 10; jj+=2)
+			{
+				a(ii, jj) = 3;
+			}
+		}
+		MultiArray<2, double> b(_spl_shp(_spl(0,2,20),_spl(0,2,10)));
+		copy_splice(_spl(0,2,20), _spl(0,2,10), a, b);
+
+		MultiArray<2, double> c(b.shape(), 3);
+		shouldEqual(b, c);
+		std::cerr << "DONE\n";
+	}
+
+	
+/** Check whether the ridge regression functor compiles and runs
+ */
+    void RFridgeRegressionTest()
+    {
+		std::cerr << "RF_ridgeRegressionTest()....";
+		std::cerr << "WARNING: THIS TEST CURRENTLY ONLY CHECKS WHETHER RIDGE REGRESSION COMPILES AND RUNS WITHOUT ERROR...";
+		int ii = data.size() - 3; // this is the pina_indians dataset
+	    //the ridge split class
+        vigra::RandomForest<> rf(vigra::RandomForestOptions().tree_count(1));
+		vigra::GiniRidgeSplit ridgeSplit;
+	    rf.learn(data.features(ii), data.labels(ii), rf_default(), ridgeSplit);
+		std::cerr << "DONE\n";
+    }
+	
+/** Tests whether RF rejects Matrices containing NaN
+ */	
+	void RF_NanCheck()
+    {
+        //Create Test output by Random Forest
+		typedef MultiArrayShape<2>::type Shp_t;
+        {
+            std::cerr << "RF_NanCheck(): checking contains_nan() and RF precondition on NaN\n";
+			MultiArray<2, double> dfeatures(Shp_t(10, 5));
+			MultiArray<2, float>  ffeatures(Shp_t(10, 5));
+			for(int ii = 0; ii < 10; ++ii)
+			{
+				for(int jj = 0; jj < 5; ++jj)
+				{
+					dfeatures(ii, jj) = (ii*jj+jj)%97;
+					ffeatures(ii, jj) = (ii*jj+jj)%97;
+				}
+			}
+			shouldEqual(detail::contains_nan(dfeatures), false);
+			shouldEqual(detail::contains_nan(ffeatures), false);
+					
+			dfeatures(2, 3) = std::numeric_limits<double>::quiet_NaN();
+			ffeatures(2, 3) = std::numeric_limits<float>::quiet_NaN();
+			
+			shouldEqual(detail::contains_nan(dfeatures), true);
+			shouldEqual(detail::contains_nan(ffeatures), true);
+			
+			for(int ii = 0; ii < 10; ++ii)
+			{
+				for(int jj = 0; jj < 5; ++jj)
+				{
+					dfeatures(ii, jj) = std::numeric_limits<double>::quiet_NaN();
+					ffeatures(ii, jj) = std::numeric_limits<double>::quiet_NaN();
+				}
+			}
+			
+			shouldEqual(detail::contains_nan(dfeatures), true);
+			shouldEqual(detail::contains_nan(ffeatures), true);
+			
+			for(int ii = 0; ii < 10; ++ii)
+			{
+				for(int jj = 0; jj < 5; ++jj)
+				{
+					dfeatures(ii, jj) = (ii*jj+jj)%97;
+					ffeatures(ii, jj) = (ii*jj+jj)%97;
+				}
+			}
+			shouldEqual(detail::contains_nan(dfeatures), false);
+			shouldEqual(detail::contains_nan(ffeatures), false);
+					
+			dfeatures(2, 3) = std::numeric_limits<double>::signaling_NaN();
+			ffeatures(2, 3) = std::numeric_limits<double>::signaling_NaN();
+			
+			shouldEqual(detail::contains_nan(dfeatures), true);
+			shouldEqual(detail::contains_nan(ffeatures), true);
+			
+			for(int ii = 0; ii < 10; ++ii)
+			{
+				for(int jj = 0; jj < 5; ++jj)
+				{
+					dfeatures(ii, jj) = std::numeric_limits<double>::signaling_NaN();
+					ffeatures(ii, jj) = std::numeric_limits<double>::signaling_NaN();
+				}
+			}
+			
+			shouldEqual(detail::contains_nan(dfeatures), true);
+			shouldEqual(detail::contains_nan(ffeatures), true);
+        }
+		MultiArray<2, double> dfeatures(Shp_t(10, 5));
+		MultiArray<2, double> dlabels(Shp_t(10, 1));
+		for(int ii = 0; ii < 10; ++ii)
+		{
+			for(int jj = 0; jj < 5; ++jj)
+			{
+				dfeatures(ii, jj) = (ii*jj+jj)%97;
+			}
+			dlabels(ii, 0) = ii%2;
+		}
+		dlabels(9,0) = std::numeric_limits<double>::quiet_NaN();
+		RandomForest<> rf;
+		try
+		{
+			rf.learn(dfeatures, dlabels);
+			shouldEqual(0, 1);
+		}
+		catch( ... )
+		{
+		}
+		dfeatures(9,0) = dlabels(9,0);
+		dlabels(9,0) = 9%2;
+		try
+		{
+			rf.learn(dfeatures, dlabels);
+			shouldEqual(0, 1);
+		}
+		catch( ... )
+		{
+		}
+
+    }
 /**
         ClassifierTest::RFdefaultTest():
     Learns The Refactored Random Forest with a fixed Random Seed and default sampling Options on
@@ -280,12 +522,14 @@ struct ClassifierTest
                 double oob = 0.0;
                 for(int jj = 0; jj < 10; ++jj)
                 {
+                    rf::visitors::OOB_PerTreeError oob_v;
                     vigra::RandomForest<> RF2(vigra::RandomForestOptions()
 												 .tree_count(100));
 
-                     oob  += RF2.learn(  data.features(ii),
-                                            data.labels(ii));
-                    //std::cerr << oob <<" " <<  data.oobError(ii) <<std::endl;
+                     RF2.learn(  data.features(ii), 
+                                 data.labels(ii),
+                                 rf::visitors::create_visitor(oob_v));
+                     oob += oob_v.oobError;
                 }
                 oob = oob/10;
                 oob = oob - data.oobError(ii);
@@ -381,10 +625,11 @@ struct ClassifierTest
         std::for_each(features.begin(), features.end(),UnaryRandomFunctor());
         std::cerr << "RFnoiseTest(): Learning 100 Trees on Noise.";
         vigra::RandomForest<> RF2(vigra::RandomForestOptions().tree_count(100));
-        double oob = RF2.learn(features, labels);
+        rf::visitors::OOB_PerTreeError oob_v;
+        RF2.learn(features, labels,create_visitor(oob_v));
         std::cerr << "DONE!\n";
         std::cerr << "RFnoiseTest(): Comparing oob with 0.5:";
-        shouldEqualTolerance(oob, 0.5, 0.01);
+        shouldEqualTolerance(oob_v.oobError, 0.5, 0.01);
         std::cerr << "DONE!\n\n";
     }
 
@@ -407,7 +652,7 @@ struct ClassifierTest
 
 			int ii = data.size() - 3; // this is the pina_indians dataset
             {
-            	vigra::VariableImportanceVisitor var_imp;
+            	vigra::rf::visitors::VariableImportanceVisitor var_imp;
 
                 vigra::RandomForest<>
 					RF2(vigra::RandomForestOptions().tree_count(255));
@@ -515,6 +760,7 @@ struct ClassifierTestSuite
     ClassifierTestSuite()
     : vigra::test_suite("ClassifierTestSuite")
     {
+	
         add( testCase( &ClassifierTest::RFdefaultTest));
 #ifndef FAST
         add( testCase( &ClassifierTest::RFsetTest));
@@ -522,9 +768,14 @@ struct ClassifierTestSuite
         add( testCase( &ClassifierTest::RFoobTest));
         add( testCase( &ClassifierTest::RFnoiseTest));
         add( testCase( &ClassifierTest::RFvariableImportanceTest));
-        add( testCase( &ClassifierTest::RFwrongLabelTest));
+        add( testCase( &ClassifierTest::RF_NanCheck));
+        add( testCase( &ClassifierTest::RF_InfCheck));
+        add( testCase( &ClassifierTest::RF_SpliceTest));
+        add( testCase( &ClassifierTest::RF_AlgorithmTest));
 #endif
         add( testCase( &ClassifierTest::RFresponseTest));
+		
+        add( testCase( &ClassifierTest::RFridgeRegressionTest));
 #ifdef HasHDF5
 		add( testCase( &ClassifierTest::HDF5ImpexTest));
 #endif
