@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*                 Copyright 2004 by Ullrich Koethe                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -292,7 +291,7 @@ struct CompareFunctor
 
 	CompareFunctor(): sumDifference_(0) {}
 
-	void operator()(const float &a, const float &b)
+	void operator()(const fftw_real &a, const fftw_real &b)
 		{ sumDifference_+= abs(a-b); }
 
     double operator()()
@@ -303,7 +302,7 @@ struct GaborTests
 {
 	ImageImportInfo info;
 	int w, h;
-	FImage image;
+	BasicImage<fftw_real> image;
 
 	GaborTests()
 		: info("ghouse.gif"),
@@ -314,7 +313,7 @@ struct GaborTests
 	}
 
 	template<class Iterator, class Accessor>
-	void checkImage(triple<Iterator, Iterator, Accessor> src, char *filename)
+	void checkImage(triple<Iterator, Iterator, Accessor> src, const char *filename)
 	{
 		if (initializing)
 		{
@@ -337,7 +336,7 @@ struct GaborTests
 
 	void testImages()
 	{
-		FImage filter(w, h);
+		BasicImage<fftw_real> filter(w, h);
 		int directionCount= 8;
 		int dir= 1, scale= 1;
 		double angle = dir * M_PI / directionCount;
@@ -354,7 +353,7 @@ struct GaborTests
 		applyFourierFilter(srcImageRange(image), srcImage(filter), destImage(result));
 		checkImage(srcImageRange(result, FFTWMagnitudeAccessor()), "gaborresult.xv");
 
-		FImage realPart(w, h);
+		BasicImage<fftw_real> realPart(w, h);
 		applyFourierFilter(srcImageRange(image), srcImage(filter), destImage(realPart));
 
 		CompareFunctor cmp;
@@ -376,8 +375,8 @@ struct GaborTests
 		shouldEqualTolerance(cmp(), 0.0, 1e-4);
 
 		cout << "applying on ROI...\n";
-		FImage bigImage(w+20, h+20);
-		FImage::Iterator bigUL= bigImage.upperLeft() + Diff2D(5, 5);
+		BasicImage<fftw_real> bigImage(w+20, h+20);
+		BasicImage<fftw_real>::Iterator bigUL= bigImage.upperLeft() + Diff2D(5, 5);
 		copyImage(srcImageRange(image), destIter(bigUL));
 		applyFourierFilter(srcIterRange(bigUL, bigUL + Diff2D(w, h)),
 						   srcImage(filter), destImage(result));
@@ -430,18 +429,18 @@ struct GaborTestSuite
 
 int main(int argc, char **argv)
 {
-    initializing= argc>1; // global variable,
+    initializing= argc>1 && std::string(argv[1]) == "init"; // global variable,
     // initializing means: don't compare with image files but create them
 
     FFTWTestSuite fftwTest;
 
-    int failed = fftwTest.run();
+    int failed = fftwTest.run(vigra::testsToBeExecuted(argc, argv));
 
     std::cout << fftwTest.report() << std::endl;
 
     GaborTestSuite gaborTest;
 
-    failed = failed || gaborTest.run();
+    failed = failed || gaborTest.run(vigra::testsToBeExecuted(argc, argv));
 
     std::cout << gaborTest.report() << std::endl;
 
