@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*                 Copyright 2004 by Ullrich Koethe                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -148,11 +147,27 @@ struct FunctorExpressionTest
     void testIfThenElse()
     {
         should(exec(ifThenElse(Arg1(), Arg2(), Arg3()), true, 2, 3.5) == 2.0); 
-        should(exec(ifThenElse(Arg1(), Arg2(), Arg3()), false, 2, 3.5) == 3.5); 
+        should(exec(ifThenElse(Arg1(), Arg2(), Arg3()), false, 2, 3.5) == 3.5);
+        int trueBranch = 0, falseBranch = 0;
+        should(exec(ifThenElse(Arg1(), 
+                               (Var(trueBranch)  += Param(1), Arg2()),
+                               (Var(falseBranch) += Param(1), Arg3())), 
+                    true, 1, 2) == 1);
+        should(exec(ifThenElse(Arg1(), 
+                               (Var(trueBranch)  += Param(1), Arg2()),
+                               (Var(falseBranch) += Param(1), Arg3())), 
+                    true, 3.5, 2) == 3.5);
+        should(exec(ifThenElse(Arg1(), 
+                               (Var(trueBranch)  += Param(1), Arg2()),
+                               (Var(falseBranch) += Param(1), Arg3())), 
+                    false, 1, 2) == 2);
+        shouldEqual(trueBranch, 2);
+        shouldEqual(falseBranch, 1);
     }
     
     void testUnary()
     {
+        shouldEqual(exec(sq(Arg1()), 7.0) , vigra::sq(7.0)); 
         shouldEqual(exec(sqrt(Arg1()), 7.0) , VIGRA_CSTD::sqrt(7.0)); 
         shouldEqual(exec(exp(Arg1()), 4.0) , VIGRA_CSTD::exp(4.0)); 
         shouldEqual(exec(log(Arg1()), 4.0) , VIGRA_CSTD::log(4.0)); 
@@ -164,8 +179,8 @@ struct FunctorExpressionTest
         shouldEqual(exec(tan(Arg1()), 4.0) , VIGRA_CSTD::tan(4.0)); 
         shouldEqual(exec(atan(Arg1()), 0.5) , VIGRA_CSTD::atan(0.5)); 
         shouldEqual(exec(abs(Arg1()), -0.5) , 0.5); 
-//        shouldEqual(exec(norm(Arg1()), -0.5) , 0.5); 
-//        shouldEqual(exec(squaredNorm(Arg1()), -0.5) , 0.25); 
+        shouldEqual(exec(norm(Arg1()), -0.5) , 0.5); 
+        shouldEqual(exec(squaredNorm(Arg1()), -0.5) , 0.25); 
         shouldEqual(exec(floor(Arg1()), -0.5) , -1.0); 
         shouldEqual(exec(ceil(Arg1()), 0.5) , 1.0); 
         shouldEqual(exec(-Arg1(), -0.5) , 0.5); 
@@ -445,11 +460,11 @@ struct FunctorTestSuite
     }
 };
 
-int main()
+int main(int argc, char ** argv)
 {
     FunctorTestSuite test;
 
-    int failed = test.run();
+    int failed = test.run(vigra::testsToBeExecuted(argc, argv));
 
     std::cout << test.report() << std::endl;
 
