@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*                 Copyright 2004 by Ullrich Koethe                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -41,9 +40,26 @@
 #include "vigra/stdimage.hxx"
 #include "vigra/impex.hxx"
 #include "unittest.hxx"
+#include "vigra/multi_array.hxx"
 
 using namespace vigra;
 
+template <class Image>
+void failCodec(Image const & img, ImageExportInfo const & info)
+{
+    try {
+        exportImage (srcImageRange (img), info);
+        failTest( "Failed to throw exception." );
+    }
+    catch( vigra::PreconditionViolation & e )
+    {
+        std::string expected = "\nPrecondition violation!\n";
+        expected += "did not find a matching codec for the given file extension";
+        const bool rc = std::strncmp( expected.c_str(), e.what(), expected.length() ) == 0;
+        should(rc);
+    }
+}
+
 class ByteImageExportImportTest
 {
     typedef vigra::BImage Image;
@@ -67,18 +83,40 @@ public:
         const std::string formats = impexListFormats();
         const std::string extensions = impexListExtensions();
 
-        shouldEqual(formats, "BMP GIF HDR JPEG PNG PNM SUN TIFF VIFF");
-        shouldEqual(extensions, "bmp gif hdr jpeg jpg pbm pgm png pnm ppm ras tif tiff xv");
+        const char * refFormats = "BMP GIF HDR "
+#if defined(HasJPEG)
+        "JPEG "
+#endif
+#if defined(HasPNG)
+        "PNG "
+#endif
+        "PNM SUN "
+#if defined(HasTIFF)
+        "TIFF "
+#endif
+        "VIFF";
+        shouldEqual(formats, refFormats);
+
+        const char * refExtensions = "bmp gif hdr "
+#if defined(HasJPEG)
+        "jpeg jpg "
+#endif
+        "pbm pgm "
+#if defined(HasPNG)
+        "png "
+#endif
+        "pnm ppm ras "
+#if defined(HasTIFF)
+        "tif tiff "
+#endif
+        "xv";
+        shouldEqual(extensions, refExtensions);
     }
 
     void testIsImage()
     {
         should(isImage("lenna.xv"));
-#ifdef _MSC_VER
-        should(!isImage("impex.vcproj"));
-#else
-        should(!isImage("Makefile"));
-#endif
+        should(!isImage("no-image.txt"));
     }
 
     void testFile (const char *filename);
@@ -113,6 +151,9 @@ public:
     void testJPEG ()
     {
         vigra::ImageExportInfo exportinfo ("res.jpg");
+#if !defined(HasJPEG)
+        failCodec(img, exportinfo);
+#else
         exportinfo.setCompression ("100");
         exportImage (srcImageRange (img), exportinfo);
 
@@ -136,11 +177,15 @@ public:
             sum += std::abs (acc (i) - acc (i1));
 
         should (sum / (info.width () * info.height ()) < 0.1);
+#endif
     }
 
     void testTIFF ()
     {
         vigra::ImageExportInfo exportinfo ("res.tif");
+#if !defined(HasTIFF)
+        failCodec(img, exportinfo);
+#else
         exportinfo.setCompression ("LZW");
         exportImage (srcImageRange (img), exportinfo);
 
@@ -161,6 +206,7 @@ public:
 
         for (; i != img.end (); ++i, ++i1)
             should (acc (i) == acc (i1));
+#endif
     }
 
     void testBMP ()
@@ -206,7 +252,11 @@ public:
 
     void testPNG ()
     {
+#if !defined(HasPNG)
+        failCodec(img, vigra::ImageExportInfo("res.png"));
+#else
         testFile ("res.png");
+#endif
     }
 
     void testSUN ()
@@ -312,7 +362,7 @@ public:
         Image::ScanOrderIterator i1 = res.begin ();
         Image::Accessor acc = ref.accessor ();
 
-        float sum = 0;
+        double sum = 0.0;
         for (; i != ref.end (); ++i, ++i1)
                 sum += (acc (i) - acc (i1)).magnitude ();
 
@@ -322,6 +372,9 @@ public:
 
     void testJPEG ()
     {
+#if !defined(HasJPEG)
+        failCodec(img, vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
+#else
         exportImage (srcImageRange (img),
                      vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
 
@@ -340,16 +393,20 @@ public:
         Image::ScanOrderIterator i1 = res.begin ();
         Image::Accessor acc = img.accessor ();
 
-        float sum = 0;
+        double sum = 0.0;
         for (; i != img.end (); ++i, ++i1)
             {
                 sum += (acc (i) - acc (i1)).magnitude ();
             }
         should (sum / (info.width () * info.height ()) < 2.0);
+#endif
     }
 
     void testTIFF ()
     {
+#if !defined(HasTIFF)
+        failCodec(img, vigra::ImageExportInfo ("res.tif").setCompression ("LZW"));
+#else
         exportImage (srcImageRange (img),
                      vigra::ImageExportInfo ("res.tif").
                      setCompression ("LZW"));
@@ -373,6 +430,7 @@ public:
             {
                 should (acc (i) == acc (i1));
             }
+#endif
     }
 
     void testBMP ()
@@ -418,7 +476,11 @@ public:
 
     void testPNG ()
     {
+#if !defined(HasPNG)
+        failCodec(img, vigra::ImageExportInfo("res.png"));
+#else
         testFile ("res.png");
+#endif
     }
 
     void testSUN ()
@@ -567,7 +629,7 @@ public:
         Image::ScanOrderIterator i1 = res.begin ();
         Image::Accessor acc = reread.accessor ();
 
-        float sum = 0;
+        double sum = 0.0;
         for (; i != reread.end (); ++i, ++i1)
             sum += std::abs (acc (i) - acc (i1));
         should (sum / (info.width () * info.height ()) < 0.1);
@@ -575,6 +637,9 @@ public:
 
     void testJPEG ()
     {
+#if !defined(HasJPEG)
+        failCodec(img, vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
+#else
         exportImage (srcImageRange (img),
                      vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
 
@@ -593,14 +658,18 @@ public:
         Image::ScanOrderIterator i1 = res.begin ();
         Image::Accessor acc = reread.accessor ();
 
-        float sum = 0;
+        double sum = 0.0;
         for (; i != reread.end (); ++i, ++i1)
             sum += std::abs (acc (i) - acc (i1));
         should (sum / (info.width () * info.height ()) < 0.1);
+#endif
     }
 
     void testTIFF ()
     {
+#if !defined(HasTIFF)
+        failCodec(img, vigra::ImageExportInfo ("res.tif").setCompression ("LZW"));
+#else
         exportImage (srcImageRange (img),
                      vigra::ImageExportInfo ("res.tif").setCompression ("LZW"));
 
@@ -620,10 +689,14 @@ public:
         Image::Accessor acc = img.accessor ();
 
         shouldEqualSequence(i, img.end(), i1);
+#endif
     }
 
     void testTIFFForcedRange ()
     {
+#if !defined(HasTIFF)
+        failCodec(img, vigra::ImageExportInfo ("res.tif").setForcedRangeMapping(0, 255, 1, 2));
+#else
         exportImage (srcImageRange (img),
                      vigra::ImageExportInfo ("res.tif").setForcedRangeMapping(0, 255, 1, 2));
 
@@ -644,6 +717,7 @@ public:
 
         for (; i != img.end (); ++i, ++i1)
             shouldEqualTolerance(acc (i) / 255.0, acc (i1) - 1.0, 1e-12);
+#endif
     }
 
     void testBMP ()
@@ -749,6 +823,9 @@ public:
 
     void testJPEG ()
     {
+#if !defined(HasJPEG)
+        failCodec(img, vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
+#else
         exportImage (srcImageRange (img),
                      vigra::ImageExportInfo ("res.jpg").setCompression ("100"));
 
@@ -771,10 +848,14 @@ public:
         for (; i != reread.end (); ++i, ++i1)
             sum += (acc (i) - acc (i1)).magnitude ();
         should (sum / (info.width () * info.height ()) < 2.0f);
+#endif
     }
 
     void testTIFF ()
     {
+#if !defined(HasTIFF)
+        failCodec(img, vigra::ImageExportInfo ("res.tif"));
+#else
         exportImage (srcImageRange (img),
                      vigra::ImageExportInfo ("res.tif"));
 
@@ -795,10 +876,14 @@ public:
 
         for (; i != img.end (); ++i, ++i1)
             shouldEqual (acc (i), acc (i1));
+#endif
     }
 
     void testTIFFForcedRange ()
     {
+#if !defined(HasTIFF)
+        failCodec(img, vigra::ImageExportInfo ("res.tif").setForcedRangeMapping(0, 255, 1, 2));
+#else
         exportImage (srcImageRange (img),
                      vigra::ImageExportInfo ("res.tif").setForcedRangeMapping(0,255,1,2));
 
@@ -823,6 +908,7 @@ public:
             shouldEqualTolerance(acc.green(i)/255.0f, acc.green(i1)-1.0f, 1e-4);
             shouldEqualTolerance(acc.blue(i)/255.0f, acc.blue(i1)-1.0f, 1e-4);
         }
+#endif
     }
 
     void testBMP ()
@@ -948,7 +1034,7 @@ public:
         {
             for(int x=0; x<2; ++x)
             {
-                img(x,y)[0] = 2*y+x + 0.5;
+                img(x,y)[0] = 2*y+x + 0.5f;
                 img(x,y)[1] = -img(x,y)[0];
                 img(x,y)[2] = 0.0;
                 img(x,y)[3] = 0.5;
@@ -961,7 +1047,8 @@ public:
         }
     }
 
-    void failingTest (char const * filename)
+    void failingTest (char const * filename, 
+                      char const * message = "exportImage(): file format does not support requested number of bands (color channels)")
     {
         try
         {
@@ -971,7 +1058,7 @@ public:
         catch( vigra::PreconditionViolation & e )
         {
             std::string expected = "\nPrecondition violation!\n";
-            expected += "exportImage(): file format does not support requested number of bands (color channels)";
+            expected += message;
             const bool rc = std::strncmp( expected.c_str(), e.what(), expected.length() ) == 0;
             should(rc);
         }
@@ -979,7 +1066,11 @@ public:
 
     void testJPEG ()
     {
+#if !defined(HasJPEG)
+        failingTest("res.jpg", "did not find a matching codec for the given file extension");
+#else
         failingTest("res.jpg");
+#endif
     }
 
     void testGIF ()
@@ -1026,6 +1117,9 @@ public:
 
     void testTIFF ()
     {
+#if !defined(HasTIFF)
+        failCodec(img, vigra::ImageExportInfo ("res.tif"));
+#else
         exportImage (srcImageRange (img),
                      vigra::ImageExportInfo ("res.tif"));
 
@@ -1044,10 +1138,14 @@ public:
 
         for (; i != img.end (); ++i, ++i1)
             shouldEqual (acc (i), acc (i1));
+#endif
     }
 
     void testPNG ()
     {
+#if !defined(HasPNG)
+        failCodec(img, vigra::ImageExportInfo("res.png"));
+#else
         exportImage (srcImageRange (img),
                      vigra::ImageExportInfo ("res.png"));
 
@@ -1068,6 +1166,7 @@ public:
         {
             should ((acc (i)- acc (i1)).magnitude() <= 1.0);
         }
+#endif
     }
 };
 
@@ -1098,7 +1197,11 @@ public:
 
     void testJPEGExport()
     {
+#if !defined(HasJPEG)
+        testExport("jpg", "did not find a matching codec for the given file extension");
+#else
         testExport("jpg");
+#endif
     }
 
     void testJPEGImport()
@@ -1110,7 +1213,11 @@ public:
 
     void testTIFFExport()
     {
+#if !defined(HasTIFF)
+        testExport("tiff", "did not find a matching codec for the given file extension");
+#else
         testExport("tiff");
+#endif
     }
 
     void testTIFFImport()
@@ -1158,7 +1265,11 @@ public:
 
     void testPNGExport()
     {
+#if !defined(HasPNG)
+        testExport("png", "did not find a matching codec for the given file extension");
+#else
         testExport("png");
+#endif
     }
 
     void testPNGImport()
@@ -1198,7 +1309,8 @@ public:
         }
     }
 
-    void testExport( const char * fext )
+    void testExport( const char * fext ,
+                     const char * message = 0)
     {
         std::string fname = "intentionalFailure/foo.";
         fname += fext;
@@ -1208,9 +1320,16 @@ public:
         }
         catch( vigra::PreconditionViolation & e ) {
             std::string expected = "\nPrecondition violation!\n";
-            expected += "Unable to open file '";
-            expected += fname;
-            expected += "'.";
+            if(message)
+            {
+                expected += message;
+            }
+            else
+            {
+                expected += "Unable to open file '";
+                expected += fname;
+                expected += "'.";
+            }
             const bool rc = std::strncmp( expected.c_str(), e.what(), expected.length() ) == 0;
             should(rc);
         }
@@ -1252,8 +1371,10 @@ struct ImageImportExportTestSuite : public vigra::test_suite
         add(testCase(&ByteRGBImageExportImportTest::testVIFF1));
         add(testCase(&ByteRGBImageExportImportTest::testVIFF2));
 
+#if defined(HasPNG)
         // 16-bit PNG
         add(testCase(&PNGInt16Test::testByteOrder));
+#endif
 
         // grayscale float images
         add(testCase(&FloatImageExportImportTest::testGIF));
@@ -1283,7 +1404,7 @@ struct ImageImportExportTestSuite : public vigra::test_suite
         add(testCase(&FloatRGBImageExportImportTest::testVIFF));
         add(testCase(&FloatRGBImageExportImportTest::testHDR));
 
-        // failure tests
+		// failure tests
         add(testCase(&ImageExportImportFailureTest::testGIFExport));
         add(testCase(&ImageExportImportFailureTest::testGIFImport));
         add(testCase(&ImageExportImportFailureTest::testJPEGExport));
@@ -1303,11 +1424,12 @@ struct ImageImportExportTestSuite : public vigra::test_suite
     }
 };
 
-int main ()
+
+int main (int argc, char ** argv)
 {
     ImageImportExportTestSuite test;
-    const int failed = test.run();
+    const int failed = test.run(vigra::testsToBeExecuted(argc, argv));
     std::cout << test.report() << std::endl;
 
-    return failed != 0;
+	return failed != 0;
 }
