@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*                 Copyright 2004 by Ullrich Koethe                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                     */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -36,17 +35,19 @@
 
 #include <iostream>
 #include <sstream>
+#include <iterator>
 #include "unittest.hxx"
 
 #include "vigra/accessor.hxx"
 #include "vigra/tinyvector.hxx"
 #include "vigra/rgbvalue.hxx"
 #include "vigra/diff2d.hxx"
+#include "vigra/box.hxx"
 
 using namespace vigra;
 
 static float di[] = {1, 2, 4 };
-static float df[] = {1.2, 2.4, 3.6 };
+static float df[] = {1.2f, 2.4f, 3.6f };
 
 template <class BVector, class IVector, class FVector>
 struct TinyVectorTest
@@ -128,10 +129,21 @@ struct TinyVectorTest
         should(!equalVector(bv3, fv3));
         should(!equalVector(iv3, fv3));
 
+        BV bv0((typename BV::value_type)0);
+        shouldEqual(bv0, BV(0,0,0));
+
         BV bv(fv3);
         should(equalIter(bv3.begin(), bv3.end(), bv.begin()));
         should(equalVector(bv3, bv));
 
+        BV bv4(bv3.begin());
+        should(equalIter(bv3.begin(), bv3.end(), bv4.begin()));
+        should(equalVector(bv3, bv4));
+
+        BV bv5(bv3.begin(), BV::ReverseCopy);
+        should(equalIter(bv3.begin(), bv3.end(),
+                         std::reverse_iterator<typename BV::iterator>(bv5.end())));
+
         FV fv(iv3);
         should(equalIter(iv3.begin(), iv3.end(), fv.begin()));
         should(equalVector(iv3, fv));
@@ -168,7 +180,7 @@ struct TinyVectorTest
         FV fvm3 = -fv3;
 
         int mi[] = { -1, -2, -4};
-        float mf[] = { -1.2, -2.4, -3.6 };
+        float mf[] = { -1.2f, -2.4f, -3.6f };
 
         should(equalIter(ivm3.begin(), ivm3.end(), mi));
         should(equalIter(fvm3.begin(), fvm3.end(), mf));
@@ -258,7 +270,7 @@ struct TinyVectorTest
         should(equalIter(bvp.begin(), bvp.end(), ip));
         should(equalIter(fvp.begin(), fvp.end(), fp));
         fvp = fv3 / 2.0;
-        float fp1[] = {0.6, 1.2, 1.8};
+        float fp1[] = {0.6f, 1.2f, 1.8f};
         should(equalIter(fvp.begin(), fvp.end(), fp1));
     }
 
@@ -559,6 +571,15 @@ struct Rect2DTest
         emptyRect.setSize(bigRect.size());
         should(bigRect == emptyRect);
     }
+
+    void testScaling()
+    {
+        shouldEqual((rect1_1 * 4).size(), Size2D(4, 4));
+        shouldEqual((rect1_1 * 4).lowerRight(), Size2D(8, 8));
+        Rect2D r2(rect1_1);
+        r2 *= 5;
+        shouldEqual(rect1_1 * 5, r2);
+    }
 };
 
 struct Rect2DTestSuite
@@ -573,24 +594,333 @@ struct Rect2DTestSuite
         add(testCase(&Rect2DTest::testIntersection));
         add(testCase(&Rect2DTest::testUnion));
         add(testCase(&Rect2DTest::testSizes));
+        add(testCase(&Rect2DTest::testScaling));
+    }
+};
+
+
+/********************************************************************/
+
+struct IBoxTest
+{
+    typedef Box<int, 2> IBox;
+
+    IBox rect1_1;
+    IBox emptyRect;
+    IBox bigRect;
+
+    typedef IBox::Vector IPoint;
+
+    IBoxTest()
+        : rect1_1(IPoint(1, 1), IPoint(2, 2)),
+          bigRect(IPoint(10, 10))
+    {
+    }
+
+    void testProperties()
+    {
+        shouldEqual(rect1_1.size()[0], 1);
+        shouldEqual(rect1_1.size()[1], 1);
+        should(!rect1_1.isEmpty());
+
+        should(emptyRect.size()[0] <= 0);
+        should(emptyRect.size()[1] <= 0);
+        should(emptyRect.isEmpty());
+
+        shouldEqual(bigRect.size()[0], 10);
+        shouldEqual(bigRect.size()[1], 10);
+        should(!bigRect.isEmpty());
+
+        should(rect1_1 != emptyRect);
+        should(bigRect != emptyRect);
+        should(bigRect != rect1_1);
+
+        bigRect = rect1_1;
+        should(bigRect == rect1_1);
+    }
+
+    void testContains()
+    {
+        should(!emptyRect.contains(IPoint(0, 0)));
+        should(!emptyRect.contains(IPoint(0, 1)));
+        should(!emptyRect.contains(IPoint(0, 2)));
+        should(!emptyRect.contains(IPoint(1, 0)));
+        should(!emptyRect.contains(IPoint(1, 1)));
+        should(!emptyRect.contains(IPoint(1, 2)));
+        should(!emptyRect.contains(IPoint(2, 0)));
+        should(!emptyRect.contains(IPoint(2, 1)));
+        should(!emptyRect.contains(IPoint(2, 2)));
+
+        should( emptyRect.contains(emptyRect));
+        should(!emptyRect.contains(rect1_1));
+        should(!emptyRect.contains(bigRect));
+
+        should(!rect1_1.contains(IPoint(0, 0)));
+        should(!rect1_1.contains(IPoint(0, 1)));
+        should(!rect1_1.contains(IPoint(0, 2)));
+        should(!rect1_1.contains(IPoint(1, 0)));
+        should( rect1_1.contains(IPoint(1, 1)));
+        should(!rect1_1.contains(IPoint(1, 2)));
+        should(!rect1_1.contains(IPoint(2, 0)));
+        should(!rect1_1.contains(IPoint(2, 1)));
+        should(!rect1_1.contains(IPoint(2, 2)));
+
+        should( rect1_1.contains(emptyRect));
+        should( rect1_1.contains(rect1_1));
+        should(!rect1_1.contains(bigRect));
+
+        should(bigRect.contains(IPoint(0, 0)));
+        should(bigRect.contains(IPoint(0, 1)));
+        should(bigRect.contains(IPoint(0, 2)));
+        should(bigRect.contains(IPoint(1, 0)));
+        should(bigRect.contains(IPoint(1, 1)));
+        should(bigRect.contains(IPoint(1, 2)));
+        should(bigRect.contains(IPoint(2, 0)));
+        should(bigRect.contains(IPoint(2, 1)));
+        should(bigRect.contains(IPoint(2, 2)));
+
+        should( bigRect.contains(emptyRect));
+        should( bigRect.contains(rect1_1));
+        should( bigRect.contains(bigRect));
+    }
+
+    void testIntersection()
+    {
+        should(!emptyRect.intersects(emptyRect));
+        should(!emptyRect.intersects(rect1_1));
+        should(!emptyRect.intersects(bigRect));
+        should(!rect1_1.intersects(emptyRect));
+        should( rect1_1.intersects(rect1_1));
+        should( rect1_1.intersects(bigRect));
+        should(!bigRect.intersects(emptyRect));
+        should( bigRect.intersects(rect1_1));
+        should( bigRect.intersects(bigRect));
+
+        should(!bigRect.intersects(IBox(IPoint(3, -3), IPoint(3, 3))));
+        should( bigRect.intersects(IBox(IPoint(3, -3), IPoint(4, 3))));
+        should( bigRect.intersects(IBox(IPoint(3, -3), IPoint(14, 3))));
+
+        should((rect1_1 & emptyRect).isEmpty());
+        should(!(rect1_1 & bigRect).isEmpty());
+        should((rect1_1 & bigRect) == rect1_1);
+    }
+
+    void testUnion()
+    {
+        should(!(rect1_1 | emptyRect).isEmpty());
+        should((rect1_1 | emptyRect) == rect1_1);
+        should((rect1_1 | bigRect) == bigRect);
+        rect1_1 |= IPoint(3, 3);
+        shouldEqual(rect1_1.begin(), IPoint(1, 1));
+        shouldEqual(rect1_1.end(), IPoint(4, 4));
+    }
+
+    void testSizes()
+    {
+        shouldEqual(rect1_1.size(), IPoint(1, 1));
+        shouldEqual(bigRect.size(), IPoint(10, 10));
+        emptyRect.setBegin(IPoint(0, 0));
+        emptyRect.setSize(IPoint(10, 10));
+        should(bigRect == emptyRect);
+        emptyRect.addSize(IPoint(-4, -7));
+        shouldEqual(emptyRect.size(), IPoint(6, 3));
+        emptyRect.setSize(bigRect.size());
+        should(bigRect == emptyRect);
+    }
+
+    void testScaling()
+    {
+        shouldEqual((rect1_1 * 4).size(), IPoint(4, 4));
+        shouldEqual((rect1_1 * 4).end(), IPoint(8, 8));
+        IBox r2(rect1_1);
+        r2 *= 5;
+        should(rect1_1 * 5 == r2);
+        r2 /= 5;
+        should(rect1_1 == r2);
+    }
+};
+
+struct FBoxTest
+{
+    typedef Box<float, 2> FBox;
+
+    FBox rect1_1;
+    FBox emptyRect;
+    FBox bigRect;
+
+    typedef FBox::Vector FPoint;
+
+    FBoxTest()
+        : rect1_1(FPoint(1, 1), FPoint(2, 2)),
+          bigRect(FPoint(10, 10))
+    {
+    }
+
+    void testProperties()
+    {
+        shouldEqual(rect1_1.size()[0], 1);
+        shouldEqual(rect1_1.size()[1], 1);
+        should(!rect1_1.isEmpty());
+
+        should(emptyRect.size()[0] <= 0);
+        should(emptyRect.size()[1] <= 0);
+        should(emptyRect.isEmpty());
+
+        shouldEqual(bigRect.size()[0], 10);
+        shouldEqual(bigRect.size()[1], 10);
+        should(!bigRect.isEmpty());
+
+        should(rect1_1 != emptyRect);
+        should(bigRect != emptyRect);
+        should(bigRect != rect1_1);
+
+        bigRect = rect1_1;
+        should(bigRect == rect1_1);
+    }
+
+    void testContains()
+    {
+        should(!emptyRect.contains(FPoint(0, 0)));
+        should(!emptyRect.contains(FPoint(0, 1)));
+        should(!emptyRect.contains(FPoint(0, 2)));
+        should(!emptyRect.contains(FPoint(1, 0)));
+        should(!emptyRect.contains(FPoint(1, 1)));
+        should(!emptyRect.contains(FPoint(1, 2)));
+        should(!emptyRect.contains(FPoint(2, 0)));
+        should(!emptyRect.contains(FPoint(2, 1)));
+        should(!emptyRect.contains(FPoint(2, 2)));
+
+        should( emptyRect.contains(emptyRect));
+        should(!emptyRect.contains(rect1_1));
+        should(!emptyRect.contains(bigRect));
+
+        should(!rect1_1.contains(FPoint(0, 0)));
+        should(!rect1_1.contains(FPoint(0, 1)));
+        should(!rect1_1.contains(FPoint(0, 2)));
+        should(!rect1_1.contains(FPoint(1, 0)));
+        should( rect1_1.contains(FPoint(1, 1)));
+        should( rect1_1.contains(FPoint(1, 2)));
+        should(!rect1_1.contains(FPoint(1, 2.1f)));
+        should(!rect1_1.contains(FPoint(2, 0)));
+        should( rect1_1.contains(FPoint(2, 1)));
+        should(!rect1_1.contains(FPoint(2.1f, 1)));
+        should( rect1_1.contains(FPoint(2, 2)));
+
+        should( rect1_1.contains(emptyRect));
+        should( rect1_1.contains(rect1_1));
+        should(!rect1_1.contains(bigRect));
+
+        should(bigRect.contains(FPoint(0, 0)));
+        should(bigRect.contains(FPoint(0, 1)));
+        should(bigRect.contains(FPoint(0, 2)));
+        should(bigRect.contains(FPoint(1, 0)));
+        should(bigRect.contains(FPoint(1, 1)));
+        should(bigRect.contains(FPoint(1, 2)));
+        should(bigRect.contains(FPoint(2, 0)));
+        should(bigRect.contains(FPoint(2, 1)));
+        should(bigRect.contains(FPoint(2, 2)));
+
+        should( bigRect.contains(emptyRect));
+        should( bigRect.contains(rect1_1));
+        should( bigRect.contains(bigRect));
+    }
+
+    void testIntersection()
+    {
+        should(!emptyRect.intersects(emptyRect));
+        should(!emptyRect.intersects(rect1_1));
+        should(!emptyRect.intersects(bigRect));
+        should(!rect1_1.intersects(emptyRect));
+        should( rect1_1.intersects(rect1_1));
+        should( rect1_1.intersects(bigRect));
+        should(!bigRect.intersects(emptyRect));
+        should( bigRect.intersects(rect1_1));
+        should( bigRect.intersects(bigRect));
+
+        should( bigRect.intersects(FBox(FPoint(3, -3), FPoint(4, 3))));
+        should( bigRect.intersects(FBox(FPoint(3, -3), FPoint(14, 3))));
+
+        should((rect1_1 & emptyRect).isEmpty());
+        should(!(rect1_1 & bigRect).isEmpty());
+        should((rect1_1 & bigRect) == rect1_1);
+    }
+
+    void testUnion()
+    {
+        should(!(rect1_1 | emptyRect).isEmpty());
+        should((rect1_1 | emptyRect) == rect1_1);
+        should((rect1_1 | bigRect) == bigRect);
+        rect1_1 |= FPoint(3, 3);
+        shouldEqual(rect1_1.begin(), FPoint(1, 1));
+        shouldEqual(rect1_1.end(), FPoint(3, 3));
+    }
+
+    void testSizes()
+    {
+        shouldEqual(rect1_1.size(), FPoint(1, 1));
+        shouldEqual(bigRect.size(), FPoint(10, 10));
+        emptyRect.setBegin(FPoint(0, 0));
+        emptyRect.setSize(FPoint(10, 10));
+        should(bigRect == emptyRect);
+        emptyRect.addSize(FPoint(-4, -7));
+        shouldEqual(emptyRect.size(), FPoint(6, 3));
+        emptyRect.setSize(bigRect.size());
+        should(bigRect == emptyRect);
+    }
+
+    void testScaling()
+    {
+        shouldEqual((rect1_1 * 4).size(), FPoint(4, 4));
+        shouldEqual((rect1_1 * 4).end(), FPoint(8, 8));
+        FBox r2(rect1_1);
+        r2 *= 5;
+        should(rect1_1 * 5 == r2);
+        r2 /= 5;
+        should(rect1_1 == r2);
+    }
+};
+
+struct BoxTestSuite
+: public test_suite
+{
+    BoxTestSuite()
+    : test_suite("BoxTestSuite")
+    {
+        add(testCase(&IBoxTest::testProperties));
+        add(testCase(&IBoxTest::testContains));
+        add(testCase(&IBoxTest::testIntersection));
+        add(testCase(&IBoxTest::testUnion));
+        add(testCase(&IBoxTest::testSizes));
+        add(testCase(&IBoxTest::testScaling));
+
+        add(testCase(&FBoxTest::testProperties));
+        add(testCase(&FBoxTest::testContains));
+        add(testCase(&FBoxTest::testIntersection));
+        add(testCase(&FBoxTest::testUnion));
+        add(testCase(&FBoxTest::testSizes));
+        add(testCase(&FBoxTest::testScaling));
     }
 };
 
 /********************************************************************/
 
-int main()
+int main(int argc, char ** argv)
 {
     PixelTypesTestSuite test;
-    int failed = test.run();
+    int failed = test.run(vigra::testsToBeExecuted(argc, argv));
     std::cout << test.report() << std::endl;
 
     Point2DTestSuite test2;
-    failed += test2.run();
+    failed += test2.run(vigra::testsToBeExecuted(argc, argv));
     std::cout << test2.report() << std::endl;
 
     Rect2DTestSuite test3;
-    failed += test3.run();
+    failed += test3.run(vigra::testsToBeExecuted(argc, argv));
     std::cout << test3.report() << std::endl;
 
+    BoxTestSuite test4;
+    failed += test4.run(vigra::testsToBeExecuted(argc, argv));
+    std::cout << test4.report() << std::endl;
+
     return (failed != 0);
 }
