@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*                 Copyright 2004 by Ullrich Koethe                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -174,12 +173,12 @@ struct ArrayVectorTest
 
         SequenceAccessor<Vector> sqa;
         SequenceAccessor<const Vector> sqca;
-        shouldEqual(sqa.size(v), 3);
-        shouldEqual(sqa.size(v + 1), 3);
-        shouldEqual(sqa.size(v, 2), 3);
-        shouldEqual(sqca.size(cv), 3);
-        shouldEqual(sqca.size(cv + 1), 3);
-        shouldEqual(sqca.size(cv, 2), 3);
+        shouldEqual(sqa.size(v), 3u);
+        shouldEqual(sqa.size(v + 1), 3u);
+        shouldEqual(sqa.size(v, 2), 3u);
+        shouldEqual(sqca.size(cv), 3u);
+        shouldEqual(sqca.size(cv + 1), 3u);
+        shouldEqual(sqca.size(cv, 2), 3u);
         should(sqa.end(v) == v[0].end());
         should(sqa.end(v + 1) == v[1].end());
         should(sqa.end(v, 2) == v[2].end());
@@ -207,7 +206,7 @@ struct ArrayVectorTest
     {
         static value_type data[] = { 0, 1, 2, 3, 4 };
         
-        shouldEqual(vector_.size(), 0);
+        shouldEqual(vector_.size(), 0u);
         
         Accessor a;
         copyLine(data, data + 5, a, std::back_inserter(vector_), a);
@@ -215,6 +214,12 @@ struct ArrayVectorTest
         shouldEqual(vector_.size(), 5);
         shouldEqualSequence(vector_.begin(), vector_.end(), data);
     }
+
+    void testAmbiguousConstructor()
+    {
+        ArrayVector<ptrdiff_t> a(2, ptrdiff_t(1));
+        ArrayVector<ptrdiff_t> b(a.begin(), a.end());
+    }
 };
 
 struct SizedIntTest
@@ -232,6 +237,78 @@ struct SizedIntTest
     }
 };
 
+struct MetaprogrammingTest
+{
+    struct TrueResult {};
+    struct FalseResult {};
+    
+    struct Derived 
+    : public TrueResult 
+    {
+        typedef TrueResult result_type;
+        typedef TrueResult value_type;
+    };
+    
+    void testInt()
+    {
+        shouldEqual(MetaInt<1>::value, 1);
+        shouldEqual((MetaMax<1,2>::value), 2);
+        shouldEqual((MetaMin<1,2>::value), 1);
+    }
+ 
+    void testLogic()
+    {
+        shouldEqual(VigraTrueType::value, true);
+        shouldEqual(VigraFalseType::value, false);        
+        should(typeid(Not<VigraFalseType>::type) == typeid(VigraTrueType));
+        should(typeid(Not<VigraTrueType>::type) == typeid(VigraFalseType));
+        should(typeid(And<VigraTrueType, VigraTrueType>::type) == typeid(VigraTrueType));
+        should(typeid(And<VigraTrueType, VigraFalseType>::type) == typeid(VigraFalseType));
+        should(typeid(And<VigraFalseType, VigraTrueType>::type) == typeid(VigraFalseType));
+        should(typeid(And<VigraFalseType, VigraFalseType>::type) == typeid(VigraFalseType));
+        should(typeid(Or<VigraTrueType, VigraTrueType>::type) == typeid(VigraTrueType));
+        should(typeid(Or<VigraTrueType, VigraFalseType>::type) == typeid(VigraTrueType));
+        should(typeid(Or<VigraFalseType, VigraTrueType>::type) == typeid(VigraTrueType));
+        should(typeid(Or<VigraFalseType, VigraFalseType>::type) == typeid(VigraFalseType));
+        should(typeid(If<VigraTrueType, TrueResult, FalseResult>::type) == typeid(TrueResult));
+        should(typeid(If<VigraFalseType, TrueResult, FalseResult>::type) == typeid(FalseResult));
+        should(typeid(IfBool<true, TrueResult, FalseResult>::type) == typeid(TrueResult));
+        should(typeid(IfBool<false, TrueResult, FalseResult>::type) == typeid(FalseResult));
+    }
+ 
+    void testTypeTools()
+    {
+        should(typeid(IsSameType<TrueResult, TrueResult>::type) == typeid(VigraTrueType));
+        should(typeid(IsSameType<TrueResult, FalseResult>::type) == typeid(VigraFalseType));
+        should(typeid(IsDifferentType<TrueResult, TrueResult>::type) == typeid(VigraFalseType));
+        should(typeid(IsDifferentType<TrueResult, FalseResult>::type) == typeid(VigraTrueType));
+        should(typeid(IsConvertibleTo<int, double>::type) == typeid(VigraTrueType));
+        should(typeid(IsConvertibleTo<int, FalseResult>::type) == typeid(VigraFalseType));
+        should(typeid(IsDerivedFrom<Derived, TrueResult>::type) == typeid(VigraTrueType));
+        should(typeid(IsDerivedFrom<Derived, FalseResult>::type) == typeid(VigraFalseType));
+        should(typeid(has_result_type<Derived>::type) == typeid(VigraTrueType));
+        should(typeid(has_result_type<FalseResult>::type) == typeid(VigraFalseType));
+        should(typeid(has_value_type<Derived>::type) == typeid(VigraTrueType));
+        should(typeid(has_value_type<FalseResult>::type) == typeid(VigraFalseType));
+
+        should(typeid(IsIterator<std::reverse_iterator<int*> >::type) == typeid(VigraTrueType));
+        should(typeid(IsIterator<int*>::type) == typeid(VigraTrueType));
+        should(typeid(IsIterator<int const*>::type) == typeid(VigraTrueType));
+        should(typeid(IsIterator<FalseResult>::type) == typeid(VigraFalseType));
+
+        should(typeid(UnqualifiedType<int>::type) == typeid(int));
+        should(typeid(UnqualifiedType<const int>::type) == typeid(int));
+        should(typeid(UnqualifiedType<int*>::type) == typeid(int));
+        should(typeid(UnqualifiedType<const int*>::type) == typeid(int));
+        should(typeid(UnqualifiedType<int&>::type) == typeid(int));
+        should(typeid(UnqualifiedType<const int&>::type) == typeid(int));
+        should(typeid(UnqualifiedType<int**>::type) == typeid(int));
+        should(typeid(UnqualifiedType<const int**>::type) == typeid(int));
+        should(typeid(UnqualifiedType<int*&>::type) == typeid(int));
+        should(typeid(UnqualifiedType<const int*&>::type) == typeid(int));
+    }
+};
+
 struct UtilitiesTestSuite
 : public vigra::test_suite
 {
@@ -240,15 +317,19 @@ struct UtilitiesTestSuite
     {
         add( testCase( &ArrayVectorTest::testAccessor));
         add( testCase( &ArrayVectorTest::testBackInsertion));
+        add( testCase( &ArrayVectorTest::testAmbiguousConstructor));
         add( testCase( &SizedIntTest::testSizedInt));
+        add( testCase( &MetaprogrammingTest::testInt));
+        add( testCase( &MetaprogrammingTest::testLogic));
+        add( testCase( &MetaprogrammingTest::testTypeTools));
     }
 };
 
-int main()
+int main(int argc, char ** argv)
 {
     UtilitiesTestSuite test;
 
-    int failed = test.run();
+    int failed = test.run(vigra::testsToBeExecuted(argc, argv));
 
     std::cout << test.report() << std::endl;
 
