@@ -320,7 +320,7 @@ pythonWatersheds2D(NumpyArray<2, Singleband<PixelType> > image,
         method[k] = (std::string::value_type)tolower(method[k]);
     
     bool haveSeeds = seeds.hasData();
-    unsigned int maxRegionLabel;
+    unsigned int maxRegionLabel = 0;
     
     if(method == "")
         method = "regiongrowing";
@@ -628,7 +628,7 @@ void defineSegmentation()
         "Parameters:\n\n"
         " image:\n"
         "    the image or volume containing the boundary indicator values "
-        "    (high values = high edgeness)\n"
+        "    (high values = high edgeness, dtype=numpy.uint8 or numpy.float32).\n"
         " neighborhood:\n"
         "    the pixel neighborhood to be used. Feasible values depend on the "
         "    dimension and method:\n\n"
@@ -637,7 +637,8 @@ void defineSegmentation()
         "      3-dimensional data:\n"
         "        6 (default) or 26\n\n"
         " seeds:\n"
-        "    a label image specifying region seeds, only supported by method 'RegionGrowing'\n" 
+        "    a label image specifying region seeds, only supported by method 'RegionGrowing' "
+        "    (with dtype=numpy.uint32).\n" 
         " method:\n"
         "    the algorithm to be used for watershed computation. Possible values:\n\n"
         "      'RegionGrowing':\n"
